@@ -6,6 +6,8 @@ import os
 session = requests.Session()
 account_url = None
 
+valid_times = {'5minute', '10minute', 'hour', 'day', 'week', 'month', '3month', 'year', 'all'}
+
 
 def oauth(payload):
     global session
@@ -30,7 +32,8 @@ def oauth(payload):
     return r
 
 
-def login(username: str = None, password: str = None, device_token: str = 'c77a7142-cc14-4bc0-a0ea-bdc9a2bf6e68',
+def login(username: str = None, password: str = None, mfa_code: str = None,
+          device_token: str = 'c77a7142-cc14-4bc0-a0ea-bdc9a2bf6e68',
           bearer_token: str = None, no_input: bool = False) -> str:
     """generates and returns OAuth2 bearer token"""
     global session
@@ -69,6 +72,8 @@ def login(username: str = None, password: str = None, device_token: str = 'c77a7
         'username': username,
         'password': password
     }
+    if mfa_code is not None:
+        payload['mfa_code'] = mfa_code
 
     r = oauth(payload)
     if r.status_code == 400:
@@ -112,8 +117,16 @@ def login(username: str = None, password: str = None, device_token: str = 'c77a7
 
     elif r.status_code == 401:
         raise RuntimeError('Invalid bearer token.')
-
-    return r.json()['access_token']
+    r = r.json()
+    if r.get('mfa_required'):
+        if no_input:
+            raise RuntimeError('Multi-factor authentication is enabled. "mfa_code" required.')
+        else:
+            mfa_code = input('Enter the code generated by your authentication app: ')
+            return login(username=username, password=password, mfa_code=mfa_code, device_token=device_token,
+                         bearer_token=bearer_token, no_input=no_input)
+    else:
+        return r['access_token']
 
 
 def is_logged_in() -> bool:
@@ -177,7 +190,7 @@ def options_positions(nonzero: bool = True) -> dict:
 
 
 def live(account_number, span: str = 'day') -> dict:
-    if span not in {'day', 'week', 'month', 'year', 'all'}:
+    if span not in valid_times:
         raise RuntimeError(f"'{span}' is not valid as span.")
     url = 'https://api.robinhood.com/historical/portfolio_v2/live/'
     r = session.get(url, params={
@@ -200,6 +213,18 @@ def quotes(instrument) -> dict:
     return r.json()
 
 
+def historicals(instrument: str, bounds: str = 'regular', interval: str = '5minute', span: str = 'day') -> dict:
+    if bounds not in {'regular', 'trading', 'extended'}:
+        raise RuntimeError(f"'{bounds}' is not valid as bounds.")
+    if interval not in valid_times:
+        raise RuntimeError(f"'{interval}' is not valid as interval.")
+    if span not in valid_times:
+        raise RuntimeError(f"'{span}' is not valid as span.")
+    url = f'https://api.robinhood.com/marketdata/historicals/{instrument}/?bounds={bounds}&interval={interval}'
+    r = session.get(url)
+    return r.json()
+
+
 def orders(price, symbol, instrument=None, quantity=1, type='market', side='buy', time_in_force='gfd',
            trigger='immediate', account=None) -> dict:
     global account_url
